# cython: preliminary_late_includes_cy28=False
r"""
Conversion of small Polymake objects to Sage

The implementation is done at C++ level and should be fast enough
in practice.
"""
###############################################################################
#       Copyright (C) 2017-2019  Vincent Delecroix <vincent.delecroix@labri.fr> 
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from __future__ import absolute_import

from libcpp.vector cimport vector

from .defs cimport pm_Matrix_get, pm_Integer, pm_Rational
from .integer cimport Integer
from .rational cimport Rational
from .vector cimport VectorInteger, VectorRational
from .matrix cimport MatrixInt, MatrixFloat, MatrixInteger, MatrixRational
from .quadratic_extension cimport QuadraticExtension
from .perl_object cimport PerlObject
from .big_object import PolytopeRational, PolytopeQuadraticExtension

from .cygmp.types cimport mpz_t, mpq_t
from .cygmp.mpz cimport mpz_set, mpz_neg
from .cygmp.mpq cimport mpq_init, mpq_clear, mpq_set, mpq_numref, mpq_denref, mpq_canonicalize

from sage.all import (ZZ, QQ, AA, RIF, RDF, QuadraticField, NumberField,
        polygen, PolynomialRing, FreeModule, MatrixSpace, parent, matrix)
from sage.rings.number_field.number_field import NumberField_quadratic

from sage.libs.flint.fmpq cimport fmpq_set_mpq
from sage.libs.flint.fmpq_mat cimport fmpq_mat_entry

from sage.ext.stdsage cimport PY_NEW
from sage.structure.parent cimport Parent
from sage.rings.number_field.number_field_element_quadratic cimport NumberFieldElement_quadratic
from sage.rings.integer cimport Integer as sage_Integer
from sage.rings.rational cimport Rational as sage_Rational

from sage.modules.vector_integer_dense cimport Vector_integer_dense as sage_Vector_integer_dense
from sage.modules.vector_rational_dense cimport Vector_rational_dense as sage_Vector_rational_dense
from sage.matrix.matrix_rational_dense cimport Matrix_rational_dense as sage_Matrix_rational_dense
from sage.matrix.matrix_integer_dense cimport Matrix_integer_dense as sage_Matrix_integer_dense

def Integer_to_sage(Integer n):
    cdef sage_Integer ans = PY_NEW(sage_Integer)
    ans.set_from_mpz(<mpz_t>n.pm_obj.get_rep())
    return ans

def Rational_to_sage(Rational q):
    cdef sage_Rational ans = sage_Rational.__new__(sage_Rational)
    ans.set_from_mpq(<mpq_t>q.pm_obj.get_rep())
    return ans

class SageQuadraticFieldToPolymake(object):
    def __init__(self, K):
        if not isinstance(K, NumberField_quadratic):
            raise ValueError("K must be a quadratic field")

        self._K = K

    def __call__(self, elt):
        K = self._K
        if parent(elt) is not K:
            elt = K(elt)

        cdef mpq_t q
        cdef Rational a, b
        a = Rational.__new__(Rational)
        mpq_init(q)
        mpz_set(mpq_numref(q), (<NumberFieldElement_quadratic> elt).a)
        mpz_set(mpq_denref(q), (<NumberFieldElement_quadratic> elt).denom)
        mpq_canonicalize(q)
        a.pm_obj.set_mpq_srcptr(q)

        b = Rational.__new__(Rational)
        mpz_set(mpq_numref(q), (<NumberFieldElement_quadratic> elt).b)
        mpz_set(mpq_denref(q), (<NumberFieldElement_quadratic> elt).denom)
        if not K._standard_embedding:
            mpz_neg(mpq_numref(q), mpq_numref(q))
        mpq_canonicalize(q)
        b.pm_obj.set_mpq_srcptr(q)
        mpq_clear(q)

        return QuadraticExtension(a, b, int(K._D))

def QuadraticExtension_to_sage(QuadraticExtension z, field=None):
    cdef pm_Rational pm_r = z.pm_obj.r()
    cdef sage_Integer denom
    cdef sage_Rational a, b, r
    cdef NumberFieldElement_quadratic ans

    if pm_r == 0:
        a = sage_Rational.__new__(sage_Rational)
        mpq_set(a.value, z.pm_obj.a().get_rep())
        if field is not None:
            return field(a)
        else:
            return a
    else:
        a = sage_Rational.__new__(sage_Rational)
        b = sage_Rational.__new__(sage_Rational)
        r = sage_Rational.__new__(sage_Rational)
        mpq_set(a.value, z.pm_obj.a().get_rep())
        mpq_set(b.value, z.pm_obj.b().get_rep())
        mpq_set(r.value, pm_r.get_rep())

        if field is None and r.is_square():
            field = QQ

        if field is QQ:
            if not b:
                return a
            else:
                num_root, num_rem = r.numerator().sqrtrem()
                den_root, den_rem = r.denominator().sqrtrem()
                if num_rem or den_rem:
                    raise ValueError("not rational")
                return a + b * num_root / den_root

        if field is None:
            x = polygen(QQ, 'x')
            D = (r.numerator() * r.denominator()).squarefree_part()
            K = QuadraticField(D, 'sqrt{}'.format(D), embedding=AA(D).sqrt())
        elif not isinstance(field, NumberField_quadratic):
            raise ValueError("field must be a quadratic number field")
        else:
            K = field

        D = K._D   # should we access this hidden attribute?
        if not D.is_squarefree():
            raise RuntimeError

        q = r / D
        num_scale, num_rem = q.numerator().sqrtrem()
        den_scale, den_rem = q.denominator().sqrtrem()
        if num_rem or den_rem:
            raise ValueError("invalid Sage number field for polymake QuadraticExtension")

        b *= num_scale
        b /= den_scale

        denom = a.denominator().lcm(b.denominator())
        a *= denom
        b *= denom
        # WARNING: calling K(0) or K(1) will *always* create the same element
        # again and again (the cached zero and one respectively)
        ans = K.zero().__copy__()
        mpz_set(ans.a, mpq_numref(a.value))
        mpz_set(ans.b, mpq_numref(b.value))
        mpz_set(ans.denom, denom.value)

        if not ans.standard_embedding:
            mpz_neg(ans.b, ans.b)

        return ans

def QuadraticExtension_from_sage(NumberFieldElement_quadratic x):
    return SageQuadraticFieldToPolymake(x.parent())(x)

#def SetInt_to_sage(SetInt a):
#    pass

def VectorInteger_to_sage(VectorInteger v):
    V = FreeModule(ZZ, len(v))
    cdef sage_Vector_integer_dense ans = V.zero().__copy__()
    cdef int i
    for i in range(len(v)):
        mpz_set(ans._entries[i], v.pm_obj[i].get_rep())
    return ans

def VectorInteger_from_sage(sage_Vector_integer_dense v):
    cdef vector[pm_Integer] data
    cdef int i
    cdef pm_Integer z
    for i in range(v._degree):
        z.set_mpz_srcptr(v._entries[i])
        data.push_back(z)

    cdef VectorInteger ans = VectorInteger.__new__(VectorInteger)
    ans.pm_obj = data
    return ans

def VectorRational_to_sage(VectorRational v):
    V = FreeModule(QQ, len(v))
    cdef sage_Vector_rational_dense ans = V.zero().__copy__()
    cdef int i
    for i in range(len(v)):
        mpq_set(ans._entries[i], v.pm_obj[i].get_rep())
    return ans

def VectorRational_from_sage(sage_Vector_rational_dense v):
    cdef vector[pm_Rational] data
    cdef int i
    cdef pm_Rational q
    for i in range(v._degree):
        q.set_mpq_srcptr(v._entries[i])
        data.push_back(q)

    cdef VectorRational ans = VectorRational.__new__(VectorRational)
    ans.pm_obj = data
    return ans

def VectorQuadraticExtension_to_sage(v):
    raise NotImplementedError

def VectorQuadraticExtension_from_sage(v):
    raise NotImplementedError

def MatrixInt_to_sage(MatrixInt m):
    cdef Py_ssize_t i, j
    cdef Py_ssize_t nrows = m.pm_obj.rows()
    cdef Py_ssize_t ncols = m.pm_obj.cols()
    M = MatrixSpace(ZZ, nrows, ncols)
    cdef sage_Matrix_integer_dense ans = M.zero().__copy__()
    for i in range(nrows):
        for j in range(ncols):
            ans.set_unsafe_si(i, j, pm_Matrix_get(m.pm_obj, i, j))
    return ans

def MatrixInt_from_sage(sage_Matrix_integer_dense m):
    raise NotImplementedError

def MatrixFloat_to_sage(MatrixFloat m):
    cdef Py_ssize_t i, j
    cdef Py_ssize_t nrows = m.pm_obj.rows()
    cdef Py_ssize_t ncols = m.pm_obj.cols()
    M = MatrixSpace(RDF, nrows, ncols)
    ans = M()
    for i in range(nrows):
        for j in range(ncols):
            ans[i,j] = pm_Matrix_get(m.pm_obj, i, j)
    return ans

def MatrixFloat_from_sage(m):
    raise NotImplementedError

def MatrixInteger_to_sage(MatrixInteger m):
    cdef Py_ssize_t i, j
    cdef Py_ssize_t nrows = m.pm_obj.rows()
    cdef Py_ssize_t ncols = m.pm_obj.cols()
    M = MatrixSpace(ZZ, nrows, ncols)
    cdef sage_Matrix_integer_dense ans = M.zero().__copy__()
    for i in range(nrows):
        for j in range(ncols):
            ans.set_unsafe_mpz(i, j, <mpz_t> pm_Matrix_get(m.pm_obj, i, j).get_rep())
    return ans

def MatrixInteger_from_sage(sage_Matrix_integer_dense m):
    raise NotImplementedError

def MatrixRational_to_sage(MatrixRational m):
    cdef Py_ssize_t i, j
    cdef Py_ssize_t nrows = m.pm_obj.rows()
    cdef Py_ssize_t ncols = m.pm_obj.cols()
    M = MatrixSpace(QQ, nrows, ncols)
    cdef sage_Matrix_rational_dense ans = M.zero().__copy__()
    for i in range(nrows):
        for j in range(ncols):
            fmpq_set_mpq(fmpq_mat_entry(ans._matrix, i, j), <mpq_t> pm_Matrix_get(m.pm_obj, i, j).get_rep())
    return ans

def MatrixRational_from_sage(sage_Matrix_rational_dense m):
    raise NotImplementedError

def MatrixQuadraticExtension_to_sage(m, field=None):
    entries = [m[i,j] for i in range(m.rows()) for j in range(m.cols())]

    if field is None:
        if not entries:
            field = QQ
        else:
            D = set((x.r().numerator() * x.r().denominator()).sage().squarefree_part() for x in entries)
            D.discard(0)
            D.discard(1)
            if len(D) > 1:
                raise ValueError("D = {} consists of more than one root".format(D))
            if len(D) == 0:
                field = QQ
            else:
                D = D.pop()
                field = QuadraticField(D, 'sqrt{}'.format(D), embedding=AA(D).sqrt())

    entries = [x.sage(field) for x in entries]
    return matrix(field, m.rows(), m.cols(), entries)

def MatrixQuadraticExtension_from_sage(m):
    raise NotImplementedError

def PolytopeRational_to_sage(PerlObject pm, **opts):
    from sage.geometry.polyhedron.constructor import Polyhedron
    return Polyhedron(ieqs=pm.FACETS.sage(), eqns=pm.AFFINE_HULL.sage(), **opts)

def PolytopeQuadraticExtension_to_sage(PerlObject pm, base_ring=None, **opts):
    from sage.geometry.polyhedron.constructor import Polyhedron
    return Polyhedron(ieqs=pm.FACETS.sage(), eqns=pm.AFFINE_HULL.sage(), **opts)

def Polytope_from_sage(p):
    if p.base_ring() is ZZ or p.base_ring() is QQ:
        return PolytopeRational(
                  CONE_AMBIENT_DIM = 1 + p.parent().ambient_dim(),
                  POINTS = [[1] + v for v in p.vertices_list()] + \
                          [[0] + r for r in p.rays_list()],
                  INPUT_LINEALITY = [[0] + l for l in p.lines_list()])
    elif isinstance(p.base_ring(), NumberField_quadratic):
        f = SageQuadraticFieldToPolymake(p.base_ring())
        return PolytopeQuadraticExtension(
                CONE_AMBIENT_DIM = 1 + p.parent().ambient_dim(),
                POINTS = [[1] + list(map(f, v)) for v in p.vertices_list()] + \
                         [[0] + list(map(f, r)) for r in p.rays_list()],
                INPUT_LINEALITY = [[0] + list(map(f, l)) for l in p.lines_list()])
    else:
        raise NotImplementedError



###############################################################################
#       Copyright (C) 2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr> 
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from .defs cimport ostringstream, pm_repr_wrap

from libcpp cimport bool
from libcpp.string cimport string

cdef class SetSetInt(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        #pm_repr_wrap[pm_Set[pm_Set[int]]](out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

cdef class SetMatrixRational(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap[pm_Set[pm_Matrix[pm_Rational]]](out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

cdef class SetArrayInt(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap[pm_Set[pm_Array[int]]](out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

cdef class SetInt(PmObject):
    def __len__(self):
        return self.pm_obj.size()
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap[pm_Set[int]](out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')
    def __iter__(self):
        # TODO: Entire
        raise NotImplementedError
        #cdef pm_SetInt_iterator it = entire_SetInt(self.pm_obj)
        #while not it.at_end():
        #    yield it.get()
        #    it.next()




#!/usr/bin/env python3

import unittest
import random

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except ImportError:
    sage = None

import polymake
import polymake.matrix

try:
    import gmpy2
except ImportError:
    gmpy2 = None

class TestMatrix(unittest.TestCase):
    mats = [(int, polymake.matrix.MatrixInt),
            (float, polymake.matrix.MatrixFloat),
            (polymake.Integer, polymake.matrix.MatrixInteger),
            (polymake.Rational, polymake.matrix.MatrixRational),
            (polymake.QuadraticExtension, polymake.matrix.MatrixQuadraticExtension)]

    def test_init(self):
        m1 = polymake.matrix.MatrixInt([[1,2], [3,4]])
        m2 = polymake.matrix.MatrixInt(2, 2, [1,2,3,4])
        self.assertEqual(m1, m2)

        m1 = polymake.matrix.MatrixFloat([[1.3, 2.5, 0.2], [-1.3, 1, -1.3]])
        m2 = polymake.matrix.MatrixFloat(2, 3, [1.3, 2.5, 0.2, -1.3, 1, -1.3])
        self.assertEqual(m1, m2)

        polymake.matrix.MatrixInteger([[1,2], [-3,2**100]])
        polymake.matrix.MatrixRational([[(1,2), polymake.Rational(3,4)], [5, (-6,7)]])
        polymake.matrix.MatrixQuadraticExtension([[(1,2,3), polymake.QuadraticExtension(1,1,5)],
                                           [polymake.Integer(-3), 5**100]])

    def test_binop(self):
        m1 = polymake.matrix.MatrixInt([[1,2,3],[4,5,6]])
        m2 = polymake.matrix.MatrixInt([[-1,-1,-1],[-2,-2,-2]])
        s = polymake.matrix.MatrixInt([[0,1,2],[2,3,4]])
        self.assertEqual(type(m1 + m2), type(s))
        self.assertEqual(m1 + m2, s)

    def test_getitem(self):
        for scal_typ, mat_typ in self.mats:
            m = mat_typ([[1,2],[3,4]])
            self.assertEqual(type(m[0,0]), scal_typ)
            self.assertEqual(m[0,0], scal_typ(1))

    def test_sage(self):
        if sage is None:
            return

        for scal_typ, mat_typ in self.mats:
            m = mat_typ([[1,2],[3,4]])
            m.sage()

        m = polymake.matrix.MatrixQuadraticExtension([
            [(1,2,3),((2,3),-1,3)],[(1,1,3),2]])
        m.sage()


        m = polymake.matrix.MatrixQuadraticExtension([[(0,1,5),(1,(-1,5),5)]])
        mm = m.sage()
        self.assertEqual(mm[0,0]**2, 5)
        self.assertEqual(5*(mm[0,1]-1)**2, 1)



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from libcpp.string cimport string

from .integer cimport Integer
from .rational cimport Rational

cdef class SparseMatrixInt(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, elt):
        cdef Py_ssize_t nrows, ncols, i,j
        nrows = self.pm_obj.rows()
        ncols = self.pm_obj.cols()
        i,j = elt
        if not (0 <= i < nrows) or not (0 <= j < ncols):
            raise IndexError("matrix index out of range")

        return pm_SparseMatrix_get(self.pm_obj, i, j)

    def rows(self):
        return self.pm_obj.rows()
    def cols(self):
        return self.pm_obj.cols()

cdef class SparseMatrixRational(PmObject):
    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __getitem__(self, elt):
        cdef Py_ssize_t nrows, ncols, i,j
        nrows = self.pm_obj.rows()
        ncols = self.pm_obj.cols()
        i,j = elt
        if not (0 <= i < nrows) or not (0 <= j < ncols):
            raise IndexError("matrix index out of range")

        cdef Rational ans = Rational.__new__(Rational)
        cdef pm_Rational q = pm_SparseMatrix_get(self.pm_obj, i, j)
        ans.pm_obj.set_mpq_srcptr(q.get_rep())
        return ans

    def rows(self):
        return self.pm_obj.rows()
    def cols(self):
        return self.pm_obj.cols()





r"""
Properties of perl objects in polymake

This file defines how perl properties are handled.

.. TODO::

    Make the design flexible enough so that it is possible to redefine how are
    handled properties.
"""
###############################################################################
#       Copyright (C) 2016      Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

cpdef get_property_handler(bytes)
cpdef get_method_handler(bytes)

cdef dict handlers



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016      Vincent Delecroix <vincent.delecroix@labri.fr>
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################


from .defs cimport Main

cdef Main * pm



from .types cimport *

cdef extern from "gmp.h":

    ### Random Number Functions ###

    # Random State Initialization
    void gmp_randinit_default (gmp_randstate_t state)
    int gmp_randinit_mt (gmp_randstate_t state)
    void gmp_randinit_lc_2exp (gmp_randstate_t state, mpz_t a, unsigned long c, unsigned long m2exp)
    int gmp_randinit_lc_2exp_size (gmp_randstate_t state, unsigned long size)
    int gmp_randinit_set (gmp_randstate_t rop, gmp_randstate_t op)
    # void gmp_randinit (gmp_randstate_t state, gmp_randalg_t alg, ...)
    void gmp_randclear (gmp_randstate_t state)

    # Random State Seeding
    void gmp_randseed (gmp_randstate_t state, mpz_t seed)
    void gmp_randseed_ui (gmp_randstate_t state, unsigned long int seed)

    # Random State Miscellaneous
    unsigned long gmp_urandomb_ui (gmp_randstate_t state, unsigned long n)
    unsigned long gmp_urandomm_ui (gmp_randstate_t state, unsigned long n)



# distutils: language = c++
# distutils: libraries = polymake
###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

# This file contain header declarations from polymake (in include/core/polymake).
# The explicit list of supported "small objects" from polymake is declared
# in this file
#
#     AccurateFloat.h
#     AnyString.h
#     Array.h
#     Bitset.h
#     CascadedContainer.h
#     ContainerChain.h
#     ContainerUnion.h
#     EmbeddedList.h
#     EquivalenceRelation.h
#     FaceMap.h
#     FacetList.h
#     Fibonacci.h
#     GenericGraph.h
#     GenericIncidenceMatrix.h
#     GenericIO.h
#     GenericMatrix.h
#     GenericSet.h
#     GenericStruct.h
#     GenericVector.h
#     Graph.h
#     Heap.h
#     IncidenceMatrix.h
#     IndexedSubgraph.h
#     IndexedSubset.h
#     Integer.h
#     ListMatrix.h
#     Map.h
#     Matrix.h
#     MultiDimCounter.h
#     Plucker.h
#     Polynomial.h
#     PowerSet.h
#     PuiseuxFraction.h
#     QuadraticExtension.h
#     RandomGenerators.h
#     RandomSpherePoints.h
#     RandomSubset.h
#     RationalFunction.h
#     Rational.h
#     Ring.h
#     SelectedSubset.h
#     Series.h
#     Set.h
#     Smith_normal_form.h
#     SparseMatrix.h
#     SparseVector.h
#     TransformedContainer.h
#     TropicalNumber.h
#     Vector.h
#
#     client.h
#     color.h
#     integer_linalg.h
#     linalg.h
#     meta_function.h
#     meta_list.h
#     node_edge_incidences.h
#     numerical_functions.h
#     pair.h
#     permutations.h
#     socketstream.h
#     totally_unimodular.h
#     type_utils.h
#
# lib/callable/include/Main.h

from libcpp cimport bool
from libcpp.string cimport string
from libcpp.pair cimport pair
from libcpp.vector cimport vector

from .cygmp.types cimport mpz_t, mpq_t, mpz_srcptr, mpq_srcptr

cdef extern from "<sstream>" namespace "std":
    cdef cppclass ostringstream:
        string str()

cdef extern from "wrap.h":
    void pm_repr [T] (ostringstream, T)
    void pm_repr_wrap [T] (ostringstream, T)

###############################################################################
# Polymake mathematical types                                                 #
###############################################################################

cdef extern from "polymake/Integer.h" namespace "polymake":
    ctypedef pm_const_Integer "const Integer"
    cdef cppclass pm_Integer "Integer":
        mpz_srcptr get_rep() const
        Py_ssize_t strsize(int)
        int compare(int)

# in beta, explicit casts defined
#       long to_long()   # FIXME: this is const
#       double to_double()

# in beta, set gets renamed into copy_from
        pm_Integer set_mpz_t "copy_from" (mpz_t)
        pm_Integer& set_mpz_srcptr "copy_from" (mpz_srcptr)

# in beta, non_zero replaced by is_zero
#        bool non_zero()
        bool is_zero()

        bool operator== (pm_Integer)
        bool operator== (long)
        bool operator> (pm_Integer)
        bool operator> (long)
        bool operator< (pm_Integer)
        bool operator< (long)

        int compare(pm_Integer)
        int compare(long)

        pm_Integer operator+ (pm_Integer) except +
        pm_Integer operator+ (long) except +
        pm_Integer operator- () except +
        pm_Integer operator- (pm_Integer) except +
        pm_Integer operator- (long) except +
        pm_Integer operator* (pm_Integer) except +
        pm_Integer operator* (long) except +
        pm_Integer operator/ (pm_Integer) except +
        pm_Integer operator/ (long) except +
        pm_Integer operator% (pm_Integer) except +
        pm_Integer operator% (long) except +
        pm_Integer negate()

    bool pm_is_zero "is_zero" (const pm_Integer&)
    bool pm_is_one "is_one" (const pm_Integer&)
    pm_Integer pm_pow "pow" (const pm_Integer&, long) except +
    pm_Integer pm_pow "pow" (long a, long) except +
    pm_Integer pm_abs "abs" (const pm_Integer&) except +
    pm_Integer pm_gcd "gcd" (const pm_Integer&, const pm_Integer&) except +
    pm_Integer pm_gcd "gcd" (const pm_Integer&, long) except +
    pm_Integer pm_gcd "gcd" (long, const pm_Integer&) except +
    pm_Integer pm_lcm "lcm" (const pm_Integer&, const pm_Integer&) except +
    pm_Integer pm_lcm "lcm" (const pm_Integer&, long) except +
    pm_Integer pm_lcm "lcm" (long, const pm_Integer&) except +

    # cast
    long operator()(pm_Integer) except +

cdef extern from "polymake/Rational.h" namespace 'polymake':
    ctypedef pm_const_Rational "const Rational"
    cdef cppclass pm_Rational "Rational":
        pm_Rational()

        mpq_srcptr get_rep()
# in beta, set replaced by copy_from
        pm_Rational set_mpq_t "copy_from" (mpq_t)
        pm_Rational& set_mpq_srcptr "copy_from" (mpq_srcptr)
        pm_Rational& set_long "set" (long, long)

        pm_Rational abs()

# in beta, non_zero replaced by is_zero
#        bool non_zero()
        bool is_zero()

        bool operator== (pm_Rational)
        bool operator== (pm_Integer)
        bool operator== (long)
        bool operator< (pm_Rational)
        bool operator< (pm_Integer)
        bool operator< (long)
        bool operator> (pm_Rational)
        bool operator> (pm_Integer)
        bool operator> (long)

        int compare(pm_Rational)
        int compare(pm_Integer)
        int compare(long)

        pm_Rational operator+ (pm_Rational) except +
        pm_Rational operator+ (pm_Integer) except +
        pm_Rational operator+ (long) except +
        pm_Rational operator- () except +
        pm_Rational operator- (pm_Rational) except +
        pm_Rational operator- (pm_Integer) except +
        pm_Rational operator- (long) except +
        pm_Rational operator* (pm_Rational) except +
        pm_Rational operator* (pm_Integer) except +
        pm_Rational operator* (long) except +
        pm_Rational operator/ (pm_Rational) except +
        pm_Rational operator/ (pm_Integer) except +
        pm_Rational operator/ (long) except +

    bool pm_is_zero "is_zero" (const pm_Rational&)
    bool pm_is_one "is_one" (const pm_Rational&)
    pm_Rational pm_abs "abs" (const pm_Rational&) except +
    pm_Rational pm_pow "pow" (const pm_Rational&, long) except +
    pm_Rational pm_pow "pow" (long, long) except +

cdef extern from "polymake/Vector.h" namespace 'polymake':
    cdef cppclass pm_Vector "Vector" [E]:
        pm_Vector()
        pm_Vector(int)
        pm_Vector(int, const E&)
        pm_Vector(pm_Vector[E]&)
        pm_Vector(vector[E]&)

        bool operator==(pm_Vector[E]&)
        bool operator!=(pm_Vector[E]&)

        pm_Vector[E]& operator=(pm_Vector[E]&)
        pm_Vector[E]& operator=(vector[E]&)

        void swap(pm_Vector[E]&)

        E operator[](int)
        int size()
        void clear()
        void resize(int)

        pm_Vector[E]& operator+(const pm_Vector[E]&)
        pm_Vector[E]& operator-(const pm_Vector[E]&)
        pm_Vector[E]& negate()


cdef extern from "polymake/Array.h" namespace "polymake":
    cdef cppclass pm_Array "Array" [E]:
        # NOTE: for iterators in Cython code, you need to use the following from cython.operator
        #   dereference(foo) for *foo
        #   preincrement(foo) for ++foo
        #   postincrement(foo) for foo++
        cppclass iterator:
            E operator*()
            iterator operator++()
            bint operator==(iterator&)
            bint operator!=(iterator&)

        cppclass const_iterator:
            E operator*()
            iterator operator++()
            bint operator==(const_iterator&)
            bint operator!=(const_iterator&)

        int size()
        void clear()
        void resize(int)
        void assign(int, const E&)
        bool empty()
        E operator[] (int)
        bool operator== (const pm_Array[E]&)
        bool operator!= (const pm_Array[E]&)
        const_iterator begin()
        const_iterator end()


cdef extern from "polymake/Set.h" namespace "polymake":
    cdef cppclass pm_Set "Set" [E]:
        cppclass iterator:
            E operator*()
            iterator operator++()
            bint operator==(iterator&)
            bint operator!=(iterator&)
        void clear()
        void resize(int)
        int size()
        bool empty()
        iterator begin() const
        iterator end() const


cdef extern from "polymake/Map.h" namespace "polymake":
    cdef cppclass pm_Map "Map" [T0, T1]:
        cppclass iterator:
            pair[T0,T1] operator*()
            iterator operator++()
            bint operator==(iterator&)
            bint operator!=(iterator&)
            bool at_end()

        iterator begin()

        T1 operator[] (T0)
        int size()
        bool exists(T0&)


cdef extern from "polymake/IncidenceMatrix.h" namespace "polymake":
    cdef cppclass pm_IncidenceMatrixNonSymmetric "IncidenceMatrix<NonSymmetric>":
        pm_IncidenceMatrixNonSymmetric()
        pm_IncidenceMatrixNonSymmetric(int nr, int nc)
        int rows()
        int cols()


cdef extern from "polymake/SparseMatrix.h" namespace "polymake":
    cdef cppclass pm_SparseMatrix "SparseMatrix" [T]:
        pm_SparseMatrixInt ()
        pm_SparseMatrixInt (int nr, int nc)
        int assign (int r, int c, T val)
        int rows()
        int cols()

    # WRAP_CALL(t,i,j) -> t(i,j)
    int pm_SparseMatrix_get "WRAP_CALL" (pm_SparseMatrix[int], int, int)
    pm_Rational pm_SparseMatrix_get "WRAP_CALL" (pm_SparseMatrix[pm_Rational], int, int)


cdef extern from "polymake/Matrix.h" namespace "polymake":
    cdef cppclass pm_Matrix "Matrix" [T]:
        pm_Matrix()
        pm_Matrix(int nr, int nc)
        pm_Matrix(pm_Matrix[T]&)
        pm_Matrix(vector[T]&)

        void resize(int r, int c) except+
        int assign(int r, int c, T val)
        int rows()
        int cols()

        pm_Matrix[T]& operator= (const pm_Matrix[T]&)

        bool operator==(pm_Matrix[T]&)
        bool operator!=(pm_Matrix[T]&)

        pm_Matrix[T]& operator+ (const pm_Matrix[T]&) except+
        pm_Matrix[T]& operator- (const pm_Matrix[T]&) except+


cdef extern from "polymake/Polynomial.h" namespace "polymake":
    # TODO: use Cython support for C++ templates
    cdef cppclass UniPolynomial[Coeff, Pow]:
        int n_vars()
        int n_terms()
        bool trivial()
        bool unit()
        int deg()
        int lower_deg()

        # makes compiler crash...
        # Vector[Coeff] coefficients_as_vector()

        bool operator== (const UniPolynomial[Coeff,Pow]&)
        bool operator!= (const UniPolynomial[Coeff,Pow]&)

    cdef cppclass pm_UniPolynomialRationalInt "UniPolynomial<Rational, int>":
        int n_vars()
        int n_terms()
        bool trivial()
        bool unit()
        int deg()
        int lower_deg()

        pm_Vector[pm_Rational] coefficients_as_vector()
#        bool operator== (const UniPolynomial& other)
#        bool operator!= (const UniPolynomial& other)


cdef extern from "polymake/QuadraticExtension.h" namespace "polymake":
    cdef cppclass pm_QuadraticExtension "QuadraticExtension<Rational>":
        pm_QuadraticExtension()
        pm_QuadraticExtension(pm_Rational&) except+
        pm_QuadraticExtension(pm_Rational&, pm_Rational&, pm_Rational&) except+
        pm_QuadraticExtension(pm_QuadraticExtension&) except+
        pm_QuadraticExtension& operator+(const pm_QuadraticExtension&) except+
        pm_QuadraticExtension& operator-(const pm_QuadraticExtension&) except+
        pm_QuadraticExtension& operator*(const pm_QuadraticExtension&) except+
        pm_QuadraticExtension& operator/(const pm_QuadraticExtension&) except+
        pm_QuadraticExtension& operator-() except+
        bool operator==(const pm_QuadraticExtension&)
        bool operator<(const pm_QuadraticExtension&)
        bool operator<=(const pm_QuadraticExtension&)
        bool operator>=(const pm_QuadraticExtension&)
        bool operator>(const pm_QuadraticExtension&)

        pm_QuadraticExtension& operator= (const pm_QuadraticExtension&) except+

        pm_Rational& a()
        pm_Rational& b()
        pm_Rational& r()

    bool pm_is_zero "is_zero" (pm_QuadraticExtension&)
    bool pm_is_one "is_zero" (pm_QuadraticExtension&)


cdef extern from "polymake/RationalFunction.h" namespace "polymake":
    cdef cppclass pm_RationalFunctionRationalInt "RationalFunction<Rational, int>":
        pm_UniPolynomialRationalInt numerator()
        pm_UniPolynomialRationalInt denominator()


cdef extern from "polymake/Set.h" namespace "polymake":
    cdef cppclass pm_SetInt "Set<Int>":
        pass


cdef extern from "wrap.h":
    void pm_assign_Int "WRAP_OUT" (pm_PerlPropertyValue, int)
    void pm_assign_Matrix "WRAP_OUT" (pm_PerlPropertyValue, pm_Matrix[pm_Integer])
    void pm_assign_Matrix "WRAP_OUT" (pm_PerlPropertyValue, pm_Matrix[pm_Rational])
    void pm_assign_Matrix "WRAP_OUT" (pm_PerlPropertyValue, pm_Matrix[pm_QuadraticExtension])

    # set a single entry of a matrix
    void pm_Matrix_set[T](pm_Matrix[T]&, int, int, const T&)
    const T& pm_Matrix_get[T](const pm_Matrix[T]&, int, int)

    # the except clause below is fake
    # it is used to catch errors in PerlObject.give(), however adding
    # the except statement to the declaration of give() makes cython
    # split lines like
    #        pm_get(self.pm_obj.give(prop), pm_res)
    # and store the result of give() first. This causes problems since
    # pm_PerlPropertyValue doesn't have a default constructor.

    # WRAP_IN(x,y) x>>y
#    void pm_get_float "WRAP_IN" (pm_PerlPropertyValue, float) except +ValueError


cdef extern from "polymake/PowerSet.h" namespace "polymake":
    cdef cppclass pm_PowerSet "PowerSet" [E]:
        int size()

cdef extern from "polymake/SparseVector.h" namespace "polymake":
    pass

###############################################################################
# Polymake utilities                                                          #
###############################################################################

cdef extern from "polymake/Main.h" namespace "polymake":
    cdef cppclass Main:
        Main()
        Main(string)
        void set_application(string) except +
        void set_preference(string) except +
        void pm_include "include" (string) except +

cdef extern from "polymake/client.h":
    cdef cppclass pm_PerlPropertyValue "perl::PropertyValue":
        pass

    cdef cppclass pm_PerlObject "perl::Object":
        pm_PerlObject()
        pm_PerlObject(string) except +
        pm_PerlObject(pm_PerlObject&) except+
        bool valid()
        void VoidCallPolymakeMethod(char*) except +
        void save(char*)
        pm_PerlObject copy()


# HOW DO WE ACCESS ELEMENTS OF ARRAYS, ETC?
#        long get_long_from_int "operator[]" (int i)
#        pm_PerlObject get_PerlObject_from_int "operator[]" (int i)

        pm_PerlPropertyValue take(string&)
        pm_PerlPropertyValue give(string&) # do not add except here, see pm_get for why
        pm_PerlObjectType type()
        int exists(const string& name)
        string name()
        string description()
        bool isa(pm_PerlObjectType)
        pm_PerlObject parent()

    cdef cppclass pm_PerlObjectType "perl::ObjectType":
#        pm_PerlObjectType(AnyString&)
        string name()
#        bool isa "isa" (AnyString&)

# in beta, CallPolymakeFunction is deprecated in favor of
#   call_method and call_function
#    pm_PerlObject CallPolymakeFunction (char*) except +ValueError
#    pm_PerlObject CallPolymakeHelp "CallPolymakeFunction" \
#            (char *, char *) except +ValueError
#    pm_PerlObject CallPolymakeFunction1 "CallPolymakeFunction" \
#            (char*, int) except +ValueError
#    pm_PerlObject CallPolymakeFunction2 "CallPolymakeFunction" \
#            (char*, int, int) except +ValueError
#    pm_PerlObject CallPolymakeFunction3 "CallPolymakeFunction" \
#            (char*, int, int, int) except +ValueError

    pm_PerlObject call_function "polymake::call_function" (string) except+
    pm_PerlObject call_function "polymake::call_function" (string, int) except+
    pm_PerlObject call_function "polymake::call_function" (string, int, int) except+
    pm_PerlObject call_function "polymake::call_function" (string, int, int, int) except+
    pm_Map[string,string] call_function "polymake::call_function" (string, pm_PerlObject) except+

    pm_PerlObject* new_PerlObject_from_PerlObject "new perl::Object" (pm_PerlObject)





###############################################################################
#       Copyright (C) 2017-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from .defs cimport pm_PerlObject

def test():
    pass




# encoding: utf-8
#!/usr/bin/env python
r"""
Installation script for pypolymake

It depends on distutils
"""

from __future__ import print_function

from autogen.pm_types import pm_modules
from setuptools import setup
import distutils
from distutils.cmd import Command
from distutils.command.build_ext import build_ext as _build_ext
from setuptools.extension import Extension

import os
import os.path

DIR = os.path.dirname(__file__)
DEBUG = False

# temporary fix to
#   https://github.com/videlec/pypolymake/issues/17
cfg_vars = distutils.sysconfig.get_config_vars()
cfg_vars['CFLAGS'] = cfg_vars['CFLAGS'].replace("-Wstrict-prototypes", "")


extensions = [
    Extension("polymake.cygmp.utils", ["polymake/cygmp/utils.pyx"]),
    Extension("polymake.pm_object", ["polymake/pm_object.pyx"]),
    Extension("polymake.coercion", ["polymake/coercion.pyx"]),
    Extension("polymake.perl_object", ["polymake/perl_object.pyx"]),
    Extension("polymake.main", ["polymake/main.pyx"]),
    Extension("polymake.handlers", ["polymake/handlers.pyx"]),
    Extension("polymake.big_object", ["polymake/big_object.pyx"]),
#    Extension("polymake.function_dispatcher", ["polymake/function_dispatcher.pyx"],
#        depends = ["polymake/*.pxd", "polymake/cygmp/*"]),
]

for mod in pm_modules():
    extensions.append(Extension("polymake." + mod, ["polymake/" + mod + ".pyx"]))

try:
    import sage.all
    import sage
    import sage.env
    compile_from_sage = True
except ImportError:
    compile_from_sage = False

if compile_from_sage:
    import site
    extensions.append(
        Extension("polymake.sage_conversion", ["polymake/sage_conversion.pyx"]))

class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys
        import pytest

        # run the doctests and the tests in the tests/ repo
        sys.exit(pytest.main(["-v", os.path.join(DIR, "tests")]) or
                pytest.main(["--doctest-modules", "-v", os.path.join(DIR, "polymake")]) or
                pytest.main(["--doctest-cython", "-v", os.path.join(DIR, "polymake")]))


# Adapted from Cython's new_build_ext
class build_ext(_build_ext):
    def finalize_options(self):
        # Generate files
        from autogen import rebuild
        rebuild()

        import sys

        # Check dependencies
        try:
            from Cython.Build.Dependencies import cythonize
        except ImportError as E:
            sys.stderr.write("Error: {0}\n".format(E))
            sys.stderr.write("The installation of ppl requires Cython\n")
            sys.exit(1)

        self.distribution.ext_modules[:] = cythonize(
            self.distribution.ext_modules,
            include_path=sys.path,
            gdb_debug=DEBUG)
        _build_ext.finalize_options(self)

def read_file(filename):
    with open(os.path.join(DIR,  filename)) as f:
        return f.read().strip()

setup(
  name = "pypolymake",
  author ="Vincent Delecroix, Burcin Erocal",
  author_email = "sage-devel@googlegroups.com",
  version = read_file('VERSION'),
  description = "Python wrapper for polymake",
  long_description = read_file('README'),
  license = "GNU General Public License, version 3 or later",
  ext_modules = extensions,
  packages = ["polymake", "polymake.cygmp"],
  tests_require=['pytest', 'pytest-cython'],
  package_dir = {"polymake": "polymake",
                 "polymake.cygmp": os.path.join("polymake", "cygmp")},
  package_data = {"polymake": ["*.pxd", "*.pyx", "*.h"],
                  "polymake.cygmp": ["*.pxd", "*.pyx", "*.h"]},
  cmdclass = {'build_ext': build_ext, 'test': TestCommand}
)



cdef class PmObject(object):
    cpdef _add_(self, other)
    cpdef _sub_(self, other)
    cpdef _mul_(self, other)
    cpdef _div_(self, other)
    cpdef _floordiv_(self, other)
    cpdef _richcmp_(self, other, int op)




#!/usr/bin/env python3

import unittest
import random

# NOTE: apparently this has to be done *before* importing polymake.
# Otherwise it produces segfault...
try:
    import sage.all
    import sage
except ImportError:
    sage = None

import polymake

try:
    import gmpy2
except ImportError:
    gmpy2 = None

class TestQuadraticExtension(unittest.TestCase):
    def random(self, r=None):
        anum = random.randint(-10, 10)
        aden = random.randint(1, 10)
        a = polymake.Rational(anum, aden)
        bnum = random.randint(-10, 10)
        bden = random.randint(1, 10)
        b = polymake.Rational(bnum, bden)

        if r is None:
            rnum = random.randint(1, 100)
            rden = random.randint(1, 100)
            r = polymake.Rational(rnum, rden)

        return polymake.QuadraticExtension(a, b, r)

    def test_cmp(self):
        self.assertEqual(polymake.QuadraticExtension(1,1,5), polymake.QuadraticExtension(1,1,5))
        self.assertNotEqual(polymake.QuadraticExtension(1,1,5), polymake.QuadraticExtension(-1,1,5))
        self.assertNotEqual(polymake.QuadraticExtension(1,1,5), polymake.QuadraticExtension(1,-1,5))
        self.assertNotEqual(polymake.QuadraticExtension(0,1,3), polymake.QuadraticExtension(0,1,5))

    def test_idempotent(self):
        for a,b,r in [(1,2,3),(3,0,3),(0,3,0),(1,1,1),(2,2,2)]:
            z = polymake.QuadraticExtension(a,b,r)
            self.assertEqual(polymake.QuadraticExtension(z),  z)

    def test_with_sage_input(self):
        if sage is None:
            return

        from sage.all import Integer, Rational

        self.assertEqual(polymake.QuadraticExtension(Integer(1), Integer(1), Integer(5)),
                         polymake.QuadraticExtension(1, 1, 5))
        self.assertEqual(polymake.QuadraticExtension(Rational((2,3)), Rational((-1,3)), Rational((23,7))),
                         polymake.QuadraticExtension(polymake.Rational((2,3)),
                                                     polymake.Rational((-1,3)),
                                                     polymake.Rational((23,7))))
    def test_to_sage(self):
        if sage is None:
            return

        from sage.all import Integer, Rational, QQ, AA, polygen, NumberField, QuadraticField

        z = polymake.QuadraticExtension(2, 3, 5)
        t = z.sage()
        self.assertEqual((t - 2)**2, 45)

        a = polymake.QuadraticExtension(polymake.Rational(2,5),
                                        polymake.Rational(5,7),
                                        polymake.Rational(18,17))
        b = a.sage()
        self.assertEqual((b - QQ((2,5))) * (b - QQ((2,5))), QQ((450, 833)))
    
        # With a number field argument
        x = polygen(QQ)
        b = a.sage(NumberField(2*x**2 - 17, 't'))
        self.assertEqual((b - QQ((2,5))) * (b - QQ((2,5))), QQ((450,833)))
        b = a.sage(NumberField(x**2 + QQ((8,3))*x - QQ((290,9)), 't'))
        self.assertEqual((b - QQ((2,5))) * (b - QQ((2,5))), QQ((450,833)))

        # Check that embedding is taken into account
        a = polymake.QuadraticExtension(polymake.Rational(1,2),
                                        polymake.Rational(1,2),
                                        5)
        K = QuadraticField(5, embedding=AA(5).sqrt())
        self.assertEqual(a.sage(K), (1 + K.gen())/2)
        K = QuadraticField(5, embedding=-AA(5).sqrt())
        self.assertEqual(a.sage(K), (1 - K.gen())/2)

        # Test with rationals
        a = polymake.QuadraticExtension(1, 3, 4)
        self.assertEqual(a.sage(), 7)
        self.assertEqual(type(a.sage()), Rational)
        self.assertEqual(a.sage(QQ), QQ(7))
        a = polymake.QuadraticExtension(1, 0, 5)
        self.assertEqual(a.sage(), QQ(1))
        self.assertEqual(type(a.sage()), Rational)
        self.assertEqual(a.sage(QuadraticField(5)), 1)
        self.assertEqual(type(a.sage(QuadraticField(5))),
                sage.rings.number_field.number_field_element_quadratic.NumberFieldElement_quadratic)

    def test_from_sage(self):
        if sage is None:
            return

        from sage.all import QuadraticField, NumberField, polygen, AA, QQ
        from polymake.sage_conversion import SageQuadraticFieldToPolymake

        K1 = QuadraticField(5, embedding=AA(5).sqrt())
        K2 = QuadraticField(5, embedding=-AA(5).sqrt())

        f1 = SageQuadraticFieldToPolymake(K1)
        f2 = SageQuadraticFieldToPolymake(K2)
        self.assertEqual(f1(K1.gen()), polymake.QuadraticExtension(0,1,5))
        self.assertEqual(f2(K2.gen()), polymake.QuadraticExtension(0,-1,5))

        for _ in range(10):
            a = K1.random_element()
            b = K1.random_element()
            self.assertEqual(f1(a+b), f1(a) + f1(b))
            self.assertEqual(f1(a*b), f1(a) * f1(b))

            a = K2.random_element()
            b = K2.random_element()
            self.assertEqual(f2(a+b), f2(a) + f2(b))
            self.assertEqual(f2(a*b), f2(a) * f2(b))

        x = polygen(QQ)
        K = NumberField(x**2 - x - 1, 'a', embedding=(1 + AA(5).sqrt())/2)
        a = K.gen()
        self.assertEqual(polymake.polymake(a), polymake.QuadraticExtension((1,2),(1,2),5))

        f = SageQuadraticFieldToPolymake(K)
        for _ in range(10):
            a = K.random_element()
            b = K.random_element()
            self.assertEqual(f(a+b), f(a) + f(b))
            self.assertEqual(f(a*b), f(a) * f(b))

    def test_bool(self):
        self.assertTrue(polymake.QuadraticExtension(1))
        self.assertTrue(polymake.QuadraticExtension(0,1,2))
        self.assertFalse(polymake.QuadraticExtension(0))

    def test_init(self):
        self.assertTrue(
                polymake.QuadraticExtension(1) ==
                polymake.QuadraticExtension(1,0,1) ==
                polymake.QuadraticExtension(1,1,0))

        with self.assertRaises(ValueError):
            polymake.QuadraticExtension(1,1,-1)

        self.assertTrue(
                polymake.QuadraticExtension((1,2,5)) ==
                polymake.QuadraticExtension(1, 2, 5))
        self.assertTrue(
                polymake.QuadraticExtension(((1,7), 2, 3)) ==
                polymake.QuadraticExtension(polymake.Rational(1,7), 2, 3))

    def test_abr(self):
        z = polymake.QuadraticExtension(2, 5, 7)
        self.assertEqual(z.a(), 2)
        self.assertEqual(z.b(), 5)
        self.assertEqual(z.r(), 7)

    def test_binop(self):
        # just check that we don't get error
        for _ in range(10):
            z1 = self.random()
            z2 = self.random(r=z1.r())
            z1 + z2
            z1 - z1
            z1 * z2

            z1 + 2
            2 + z1
            2 - z1
            z1 - 2
            z1 * 2
            2 * z1
            z1 / 2
            if z1:
                2 / z1

        with self.assertRaises(ZeroDivisionError):
            z1 / polymake.QuadraticExtension(0)
        with self.assertRaises(ZeroDivisionError):
            z1 / 0

        with self.assertRaises(ValueError):
            polymake.QuadraticExtension(0,1,2) + polymake.QuadraticExtension(0,1,3)
        with self.assertRaises(ValueError):
            polymake.QuadraticExtension(0,1,2) - polymake.QuadraticExtension(0,1,3)
        with self.assertRaises(ValueError):
            polymake.QuadraticExtension(0,1,2) * polymake.QuadraticExtension(0,1,3)
        with self.assertRaises(ValueError):
            polymake.QuadraticExtension(0,1,2) / polymake.QuadraticExtension(0,1,3)

if __name__ == '__main__':
    unittest.main()



# This file was autogenerated! Do not modify!
# polynomial interfacing Polynomial.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class UniPolynomialRationalInt(PmObject):
    cdef pm_UniPolynomialRationalInt pm_obj



# This file was autogenerated! Do not modify!
# map interfacing Map.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class MapRationalRational(PmObject):
    cdef pm_Map[pm_Rational,pm_Rational] pm_obj

cdef class MapStringString(PmObject):
    cdef pm_Map[string,string] pm_obj

cdef class MapIntInt(PmObject):
    cdef pm_Map[int,int] pm_obj

cdef class MapIntegerInt(PmObject):
    cdef pm_Map[pm_Integer,int] pm_obj



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from libcpp.string cimport string

from .map cimport MapStringString
from .main cimport pm

from .handlers cimport get_property_handler, get_method_handler

cdef int DEBUG = 0

def set_debug():
    global DEBUG
    DEBUG = 1
def uset_debug():
    global DEBUG
    DEBUG = 0

# this is a bug in Cython!
def _NOT_TO_BE_USED_():
    raise ValueError

    

cpdef dict get_properties(PerlObject p):
    r"""
    Return the dictionary of properties of the perl object ``p``

    The argument must correspond to a "big object" in polymake. The result
    is a dictionary where keys are attributes and keys are types.

    EXAMPLES:

    >>> import polymake
    >>> from polymake.perl_object import get_properties
    >>> c = polymake.cube(3)
    >>> m = get_properties(c)
    """
    cdef MapStringString s = MapStringString.__new__(MapStringString)
    if DEBUG:
        print("  pypolymake debug WARNING: properties_for_object...")
    s.pm_obj = call_function(<string> "Sage::properties_for_object", p.pm_obj[0])
    cdef dict d  = s.python()
    for k in list(s.keys()):
        if d[k].startswith(b"Visual::"):
            del d[k]
    return d

cpdef dict get_methods(PerlObject p):
    r"""
    Return the dictionary of methods of the perl object ``p``

    EXAMPLES:

    >>> import polymake
    >>> from polymake.perl_object import get_methods
    >>> c = polymake.cube(3)
    >>> m = get_methods(c)
    """
    cdef MapStringString s = MapStringString.__new__(MapStringString)
    if DEBUG:
        print("  pypolymake debug WARNING: methods_for_object...")
    s.pm_obj = call_function(<string> "Sage::methods_for_object", p.pm_obj[0])
    cdef dict d = s.python()
    for k in list(s.keys()):
        if d[k].startswith(b"Visual::"):
            del d[k]
    return d

# Hand written handlers
cdef PerlObject wrap_perl_object(pm_PerlObject pm_obj):
    cdef PerlObject ans = PerlObject.__new__(PerlObject)
    # TODO: remove this custom new thing
    ans.pm_obj = new_PerlObject_from_PerlObject(pm_obj)
    ans.properties = get_properties(ans)
    ans.methods = get_methods(ans)
    return ans

def call_polymake_function(bytes app, bytes name, *args):
    from .main import pm_set_application
    pm_set_application(app)
    cdef pm_PerlObject pm_obj
    if len(args) == 0:
        pm_obj = call_function(<string> name)
    elif len(args) == 1:
        pm_obj = call_function(<string> name, <int> args[0])
    elif len(args) == 2:
        pm_obj = call_function(<string> name, <int> args[0], <int> args[1])
    elif len(args) == 3:
        pm_obj = call_function(<string> name, <int> args[0], <int> args[1], <int> args[2])
    else:
        raise NotImplementedError("can only handle 0-3 arguments")

    return wrap_perl_object(pm_obj)

cdef class PerlObject(PmObject):
    def __dealloc__(self):
        del self.pm_obj

    def __getattr__(self, name):
        cdef bytes bname = name.encode('utf-8')
        if DEBUG:
            print("  pypolymake debug WARNING: __getattr__: name = {}".format(type(self), bname))

        if bname in self.properties:
            pm_type = self.properties[bname]
            handler = get_property_handler(pm_type)
            if DEBUG:
                print("  pypolymake debug WARNING: attribute found as a property")
                print("  pypolymake debug WARNING: pm_type = {}".format(pm_type))
                print("  pypolymake debug WARNING: handler = {}".format(handler))

        elif bname in self.methods:
            pm_type = self.methods[bname]
            if not pm_type:
                raise ValueError("polymake documentation is incomplete: type of {}->{} not available. Send a request to polymake developers!".format(self, name))
            handler = get_method_handler(pm_type)
            if DEBUG:
                print("  pypolymake debug WARNING: attribute found as a method")
                print("  pypolymake debug WARNING: pm_type = {}".format(pm_type))
                print("  pypolymake debug WARNING: handler = {}".format(handler))

        else:
            raise AttributeError("{} not a registered attribute".format(name))

        return handler(self, bname)

    def __dir__(self):
        return dir(self.__class__) + [x.decode('ascii') for x in self.properties] + [x.decode('ascii') for x in self.methods]

    def _save(self, filename):
        """
        Saves this polytope to a file using polymake's representation.
        """
        self.pm_obj.save(filename)

    def type_name(self):
        r"""
        Return the name of the type of this object

        >>> import polymake
        >>> P = polymake.Polytope(POINTS=[(1,1,0),(1,2,1)])
        >>> P.type_name()
        'Polytope<Rational>'
        """
        return (<bytes> self.pm_obj.type().name()).decode('ascii')

    def __str__(self): return self.type_name()
    def __repr__(self): return self.type_name()

    def name(self):
        return (<bytes> self.pm_obj.name()).decode('ascii')

    def sage(self):
        r"""
        Converts to a Sage object
        """
        if self.pm_obj.type().name() == "Polytope<Rational>":
            from .sage_conversion import PolytopeRational_to_sage
            return PolytopeRational_to_sage(self)
        if self.pm_obj.type().name() == "Polytope<QuadraticExtension<Rational>>":
            from .sage_conversion import PolytopeQuadraticExtension_to_sage
            return PolytopeQuadraticExtension_to_sage(self)

        raise NotImplementedError("Sage conversion not implemented for {}".format(self.type_name()))




###############################################################################
#       Copyright (C) 2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from cpython.object cimport Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, Py_GE
from cpython.long cimport PyLong_AsLong

from .cygmp.types cimport mpz_t, mpq_t, mpz_srcptr, mpq_srcptr
from .cygmp.mpz cimport mpz_init, mpz_clear, mpz_set_si, mpz_set
from .cygmp.mpq cimport mpq_init, mpq_clear, mpq_canonicalize, mpq_numref, mpq_denref
from .cygmp.utils cimport mpz_set_pylong, mpz_get_pyintlong, mpz_get_bytes, mpq_get_bytes, mpz_get_d_nearest, mpz_get_pylong

from libcpp.string cimport string

from .defs cimport pm_Rational, ostringstream, pm_repr
from .rational cimport Rational

cdef class Integer(PmObject):
    r"""
    Polymake integer
    """
    def __init__(self, data):
        r"""
        >>> from polymake import Integer
        >>> Integer(2)
        2
        >>> Integer(Integer(2))
        2
        """
        cdef mpz_t z

        if isinstance(data, Integer):
            self.pm_obj.set_mpz_srcptr((<Integer>data).pm_obj.get_rep())
            return

        elif not isinstance(data, (int, long)):
            data = data.__index__()

        mpz_init(z)

        if isinstance(data, long):
            mpz_set_pylong(z, data)
        elif isinstance(data, int): # only Python 2
            mpz_set_si(z, data)

        self.pm_obj.set_mpz_srcptr(<mpz_srcptr>z)
        mpz_clear(z)

    def __mpz__(self):
        # TODO: go directly through mpz initialization
        import gmpy2
        return gmpy2.mpz(int(self))

    def __index__(self):
        return mpz_get_pyintlong(self.pm_obj.get_rep())

    def __int__(self):
        return mpz_get_pyintlong(self.pm_obj.get_rep())

    def __long__(self):
        return mpz_get_pylong(self.pm_obj.get_rep())

    def __float__(self):
        return mpz_get_d_nearest(self.pm_obj.get_rep())

    python = __index__

    def _integer_(self, R=None):
        r"""
        Conversion to Sage integer
        """
        return R(self.sage())

    def numerator(self):
        return self

    def denominator(self):
        return Integer(1)

    def __nonzero__(self):
        return not pm_is_zero(self.pm_obj)

    def __repr__(self):
        cdef ostringstream out
        pm_repr(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def __float__(self):
        return <double> self.pm_obj

    # TODO: in python 3 the management of python integers is completely wrong
    cpdef _richcmp_(self, other, int op):
        cdef int c = (<Integer>self).pm_obj.compare((<Integer>other).pm_obj)
        if c < 0:
            return op in (Py_LE, Py_LT, Py_NE)
        elif c == 0:
            return op in (Py_LE, Py_EQ, Py_GE)
        else:
            return op in (Py_GE, Py_GT, Py_NE)

    cpdef _add_(self, other):
        cdef Integer ans = Integer.__new__(Integer)
        ans.pm_obj = (<Integer>self).pm_obj + (<Integer>other).pm_obj
        return ans

    cpdef _sub_(self, other):
        cdef Integer ans = Integer.__new__(Integer)
        ans.pm_obj = (<Integer>self).pm_obj - (<Integer>other).pm_obj
        return ans

    cpdef _mul_(self, other):
        cdef Integer ans = Integer.__new__(Integer)
        ans.pm_obj = (<Integer>self).pm_obj * (<Integer>other).pm_obj
        return ans

    cpdef _floordiv_(self, other):
        cdef Integer ans = Integer.__new__(Integer)
        ans.pm_obj = (<Integer>self).pm_obj / (<Integer>other).pm_obj
        return ans

    cpdef _div_(self, other):
        cdef mpq_t q
        if not other:
            raise ZeroDivisionError
        mpq_init(q)
        mpz_set(mpq_numref(q), (<Integer> self).pm_obj.get_rep())
        mpz_set(mpq_denref(q), (<Integer> other).pm_obj.get_rep())
        mpq_canonicalize(q)
        cdef Rational ans = Rational.__new__(Rational)
        ans.pm_obj.set_mpq_srcptr(q)
        mpq_clear(q)
        return ans

    def __neg__(self):
        cdef Integer ans = Integer.__new__(Integer)
        ans.pm_obj = -self.pm_obj
        return ans

    def __pow__(self, n, mod):
        if type(self) is not Integer:
            return NotImplemented
        cdef Integer ans = Integer.__new__(Integer)
        if isinstance(n, (int, long)):
            ans.pm_obj = pm_pow((<Integer> self).pm_obj, PyLong_AsLong(n))
        elif isinstance(n, Integer):
            ans.pm_obj = pm_pow((<Integer> self).pm_obj, <long> (<Integer> n).pm_obj)
        else:
            n = Integer(n)
            ans.pm_obj = pm_pow((<Integer> self).pm_obj, <long> (<Integer> n).pm_obj)
        return ans

    def sage(self):
        r"""
        Converts to a Sage integer
        """
        from .sage_conversion import Integer_to_sage
        return Integer_to_sage(self)

import numbers
numbers.Integral.register(Integer)



###############################################################################
#       Copyright (C) 2011-2012 Burcin Erocal <burcin@erocal.org>
#                     2016-2019 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL),
#  version 3 or any later version.  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
###############################################################################

from cpython.object cimport Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, Py_GE

from .integer cimport Integer
from .rational cimport Rational

import numbers

cdef class VectorInteger(PmObject):
    r"""
    A vector of integers

    >>> import polymake
    >>> v = polymake.VectorInteger([-3,1,2**80,13])
    >>> v
    -3 1 1208925819614629174706176 13
    >>> v[0]
    -3
    >>> v[-1]
    13
    """
    def __init__(self, data=None):
        if data is None:
            return
        elif isinstance(data, numbers.Integral):
            self.pm_obj.resize(<int> data)
        elif isinstance(data, VectorInteger):
            self._init_from_pm_Vector(<VectorInteger> data)
        else:
            self._init_from_iterable(data)

    def __copy__(self):
        cdef VectorInteger other = VectorInteger.__new__(VectorInteger)
        other.pm_obj = self.pm_obj
        return other

    def _init_from_pm_Vector(self, VectorInteger v):
        self.pm_obj = v.pm_obj

    def _init_from_iterable(self, data):
        cdef vector[pm_Integer] vv
        cdef Integer z
        for i,j in enumerate(data):
            z = Integer(j)
            vv.push_back(z.pm_obj)
        self.pm_obj = vv

    def __len__(self):
        return self.pm_obj.size()

    size = __len__

    def __getitem__(self, _i):
        cdef Py_ssize_t size = self.pm_obj.size()
        cdef Py_ssize_t i = <Py_ssize_t?> _i

        if i < 0:
            i += size

        if not (0 <= i < size):
            raise IndexError("integer vector out of range")

        cdef Integer ans = Integer.__new__(Integer)
        ans.pm_obj.set_mpz_srcptr(self.pm_obj[i].get_rep())
        return ans

    cpdef _richcmp_(self, other, int op):
        if op == Py_EQ:
            return (<VectorInteger> self).pm_obj == (<VectorInteger> other).pm_obj
        elif op == Py_NE:
            return (<VectorInteger> self).pm_obj != (<VectorInteger> other).pm_obj
        else:
            raise TypeError

    cpdef _add_(self, other):
        r"""
        >>> import polymake
        >>> polymake.VectorInteger([1,3]) + polymake.VectorInteger([-3,-3])
        -2 0
        """
        cdef VectorInteger left = <VectorInteger> self
        cdef VectorInteger right = <VectorInteger> other
        cdef VectorInteger ans = VectorInteger.__new__(VectorInteger)
        ans.pm_obj = left.pm_obj + right.pm_obj
        return ans

    cpdef _sub_(self, other):
        r"""
        >>> import polymake
        >>> polymake.VectorInteger([1,3]) - polymake.VectorInteger([-3,-3])
        4 6
        """
        cdef VectorInteger left = <VectorInteger> self
        cdef VectorInteger right = <VectorInteger> other
        cdef VectorInteger ans = VectorInteger.__new__(VectorInteger)
        ans.pm_obj = left.pm_obj - right.pm_obj
        return ans

    def __neg__(self):
        cdef VectorInteger other = self.__copy__()
        other.pm_obj.negate()
        return other

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def python(self):
        return [x.python() for x in self]

    def sage(self):
        r"""
        Convert to a Sage integer vector
        """
        from .sage_conversion import VectorInteger_to_sage
        return VectorInteger_to_sage(self)

cdef class VectorRational(PmObject):
    r"""
    Vector of rationals

    >>> import polymake
    >>> v = polymake.VectorRational([1, (2,3), (-3,5)])
    >>> v
    1 2/3 -3/5
    >>> v[-1]
    -3/5
    """
    def __init__(self, data=None):
        if data is None:
            return
        elif isinstance(data, numbers.Integral):
            self.pm_obj.resize(<int> data)
        elif isinstance(data, VectorRational):
            self._init_from_pm_Vector(<VectorRational> data)
        else:
            self._init_from_iterable(data)

    def __copy__(self):
        cdef VectorRational other = VectorRational.__new__(VectorRational)
        other.pm_obj = self.pm_obj
        return other

    def _init_from_pm_Vector(self, VectorRational v):
        self.pm_obj = v.pm_obj

    def _init_from_iterable(self, data):
        cdef vector[pm_Rational] vv
        cdef Rational z
        for i,j in enumerate(data):
            z = Rational(j)
            vv.push_back(z.pm_obj)
        self.pm_obj = vv

    def __len__(self):
        return self.pm_obj.size()

    size = __len__

    def __getitem__(self, _i):
        cdef Py_ssize_t size = self.pm_obj.size()
        cdef Py_ssize_t i = <Py_ssize_t?> _i

        if i < 0:
            i += size

        if not (0 <= i < size):
            raise IndexError("integer vector out of range")

        cdef Rational ans = Rational.__new__(Rational)
        ans.pm_obj.set_mpq_srcptr(self.pm_obj[i].get_rep())
        return ans

    cpdef _richcmp_(self, other, int op):
        if op == Py_EQ:
            return (<VectorRational> self).pm_obj == (<VectorRational> other).pm_obj
        elif op == Py_NE:
            return (<VectorRational> self).pm_obj != (<VectorRational> other).pm_obj
        else:
            raise TypeError

    cpdef _add_(self, other):
        r"""
        >>> import polymake
        >>> polymake.VectorRational([1,(2,3)]) + polymake.VectorRational([-3,-3])
        -2 0
        >>> polymake.VectorRational([(2,3),(1,2)]) + polymake.VectorInteger([1,1])
        5/3 3/2
        >>> polymake.VectorInteger([1,1]) + polymake.VectorRational([(2,3),(1,2)])
        5/3 3/2
        """
        cdef VectorRational left = <VectorRational> self
        cdef VectorRational right = <VectorRational> other
        cdef VectorRational ans = VectorRational.__new__(VectorRational)
        ans.pm_obj = left.pm_obj + right.pm_obj
        return ans

    cpdef _sub_(self, other):
        r"""
        >>> import polymake
        >>> polymake.VectorRational([1,3]) - polymake.VectorRational([-3,-3])
        4 6
        """
        cdef VectorRational left = <VectorRational> self
        cdef VectorRational right = <VectorRational> other
        cdef VectorRational ans = VectorRational.__new__(VectorRational)
        ans.pm_obj = left.pm_obj - right.pm_obj
        return ans

    def __neg__(self):
        cdef VectorRational other = self.__copy__()
        other.pm_obj.negate()
        return other

    def __repr__(self):
        cdef ostringstream out
        pm_repr_wrap(out, self.pm_obj)
        return (<bytes>out.str()).decode('ascii')

    def python(self):
        return [x.python() for x in self]

    def sage(self):
        from .sage_conversion import VectorRational_to_sage
        return VectorRational_to_sage(self)



# This file was autogenerated! Do not modify!
# vector interfacing Vector.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class VectorFloat(PmObject):
    cdef pm_Vector[float] pm_obj

cdef class VectorInteger(PmObject):
    cdef pm_Vector[pm_Integer] pm_obj

cdef class VectorQuadraticExtension(PmObject):
    cdef pm_Vector[pm_QuadraticExtension] pm_obj

cdef class VectorRational(PmObject):
    cdef pm_Vector[pm_Rational] pm_obj

cdef class VectorInt(PmObject):
    cdef pm_Vector[int] pm_obj



r"""
Auto generation of Cython files that deal with polymake properties
and small types

This functions in this file are called by setup.py
"""

from __future__ import print_function

import json
import os


header = "# This file was autogenerated! Do not modify!\n"

handler = """
# {cython} {perl}
cdef extern from "wrap.h" namespace "polymake":
    void pm_give_{cython} "GIVE" ({cppcython}, pm_PerlObject*, string) except +ValueError
    void pm_call_method_{cython} "CALL_METHOD" ({cppcython}, pm_PerlObject*, string) except +ValueError

def give_{cython}(PerlObject perl_object, bytes prop):
    cdef {cython} ans = {cython}.__new__({cython})
    cdef string cprop = prop
    sig_str("error in polymake while accessing property")
    pm_give_{cython}(ans.pm_obj, perl_object.pm_obj, cprop)
    sig_off()
    return ans

def call_method_{cython}(PerlObject perl_object, bytes prop):
    cdef {cython} ans = {cython}.__new__({cython})
    cdef string cprop = prop
    sig_str("error in polymake while accessing method")
    pm_call_method_{cython}(ans.pm_obj, perl_object.pm_obj, cprop)
    sig_off()
    return ans
"""

line_give_map = "    b\"{perl}\"{spacing} : give_{cython},\n"
line_call_method_map = "    b\"{perl}\"{spacing} : call_method_{cython},\n"

decl_imports="""
from libcpp cimport bool
from libcpp.string cimport string

from .defs cimport *
from .pm_object cimport PmObject
"""

class_decl="""
cdef class {cython}({parent}):
    cdef {cppcython} pm_obj
"""

#class_def="""
#from .defs cimport pm_{cython}
#
#cdef extern from "wrap.h" namespace "polymake":
#    void pm_{cython}_repr "WRAP_wrap_OUT" (ostringstream, pm_{cython})
#
#cdef class {cython}:
#    def __repr__(self):
#        cdef ostringstream out
#        pm_{cython}_repr(out, self.pm_obj)
#        return (<bytes>out.str()).decode('ascii')
#"""

# Iteration protocol
# pytype     : SetInt
# pmperltype : Set<Int>
# pmcpptype  : Set<int>
# pysubtype  : int
#
# define a "transformer" as a class method
#     cdef {py_subtype} elt_to_python(self, {pm_subtype} x):
#         return x
# or
#     cdef {py_subtype} elt_to_python(self, {pm_subtype} x):
#         {py_subtype} y = {py_subtype}.__new__({py_subtype})
#         y.pm_obj = x
#         return y
#
# It will greatly simplifies __iter__, __reverse__ and __getitem__

# c : simple container -> defines elt_to_python
# C : complex container -> defines elt_to_python
# l : __len__
# i : __iter__
# r : __reverse__
# g : __getitem__
iter_decl="""
cdef extern from "polymake/{name}.h" namespace "polymake":
    cdef cppclass pm_{cython}_iterator "Entire<const {pmcpptype}>::iterator":
        void next "operator++" ()
        bool at_end()
        {pysubtype} get "operator*" ()
    cdef pm_{cython}_iterator entire_{cython} "entire" (pm_{cython})
"""
iter_def="""
    def __iter__(self):
        cdef pm_{cython}_iterator it = entire_{cython}(self.pm_obj)
        while not it.at_end():
            yield self.elt_to_python(it.get())
            it.next()
"""
len_def="""
    def __len__(self):
        return self.pm_obj.size()
"""
getitem_def="""
    def __getitem__(self, Py_ssize_t i):
        if i < 0:
            i += self.pm_obj.size()
        if i < 0 or i >= pm.obj_size():
            raise KeyError("{cython} index out of range")
        return self.elt_to_python(self.pm_obj.get(i))
"""

from .pm_types import pm_types, caml_to_python, python_to_caml
from collections import defaultdict
all_types = pm_types()
types_by_module = defaultdict(list)
for typ,prop in all_types.items():
    mod = prop["module"]
    if mod is not None:
        types_by_module[mod].append(typ)

# NOTE: The extra_decl below is not very elegant but allows to add extra declaration
# to some module...
extra_decl = {
    'Matrix' : 
"""
cdef class PmMatrix(PmObject):
    cpdef resize(self, int nr, int nc)
    cpdef int rows(self)
    cpdef int cols(self)

    cpdef _set_list_of_lists(self, data)
    cpdef _set_matrix(self, mat)
    cpdef _set_plain_list(self, mat)
"""
}

def write_declarations():
    for pm_mod,types in types_by_module.items():
        cy_mod = caml_to_python(pm_mod)
        spacing = " " * (20 - len(pm_mod))
        print(pm_mod + '.h', spacing, "  --> ", cy_mod + '.pxd')
        with open(os.path.join('polymake', cy_mod + '.pxd'), 'w') as output:
            output.write(header)
            output.write('# ' + cy_mod + ' interfacing ' + pm_mod + '.h\n\n')
            output.write('from .defs cimport *\n')
            output.write('from .pm_object cimport PmObject\n')
            if pm_mod in extra_decl:
                print("module '{}' has extra declarations".format(pm_mod))
                output.write(extra_decl[pm_mod])
            for typ in types:
                output.write(class_decl.format(**all_types[typ]))

def undefined_small_classes():
    import re
    class_def = re.compile('cdef class (?P<pytype>[a-zA-Z]*)\(object\)', re.MULTILINE)
    undef_classes = {}
    for pm_mod,types in types_by_module.items():
        if pm_mod != "none":
            cy_mod = caml_to_python(pm_mod)
            spacing = " " * (20 - len(pm_mod))
            filename = os.path.join('polymake', cy_mod + '.pyx')
            u = set(types)
            with open(filename) as f:
                classes = class_def.findall(f.read())
            u.difference_update(classes)
            undef_classes[pm_mod] = u
    return undef_classes

#def write_undefined_classes():
#    classes = undefined_small_classes()
#    for pm_mod, types in types_by_module.items():
#        cy_mod = caml_to_python(pm_mod)
#        filename = os.path.join('polymake', 'auto_{}.pxi'.format(cy_mod))
#        with open(filename, 'w') as output:
#            output.write(header)
#            for typ in types:
#                if typ in classes[pm_mod]:
#                    output.write(class_def.format(**all_types[typ]))

def write_definitions(filename):
    with open(filename, "w") as output:
        output.write(header)
        output.write(def_imports)
        for opts in all_types.values():
            if opts["module"] is not None:
                output.write(class_def.format(**opts))

def write_handlers(filename):
    r"""
    Write down functions that given a small type
    construct the corresponding Python object with a pointer
    to that small object
    """
    with open(filename, "w") as output:
        output.write(header)
        for opts in all_types.values():
            if opts["module"] is not None:
                output.write(handler.format(**opts))

def write_mappings(filename):
    with open(filename, "w") as output:
        output.write(header)

        output.write("cdef dict auto_property_handlers = {\n")
        for opts in all_types.values():
            if opts["module"] is not None:
                spacing = " "*(40 - len(opts['perl']))
                output.write(line_give_map.format(spacing=spacing, **opts))
        output.write("}\n")

        output.write("cdef dict auto_method_handlers = {\n")
        for opts in all_types.values():
            if opts["module"] is not None:
                spacing = " "*(40 - len(opts['perl']))
                output.write(line_call_method_map.format(spacing=spacing, **opts))
        output.write("}\n")




import unittest
import random

try:
    import sage.all
    import sage
except ImportError:
    sage = None

import polymake

class TestPolymakeRational(unittest.TestCase):
    def test_is_sage_type(self):
        if sage is None:
            return

        from polymake.coercion import is_sage_type

        for a in [1, 'hello', int, polymake.Integer(3)]:
            self.assertFalse(is_sage_type(type(a)))

        for a in [sage.all.Integer(1),
                  sage.all.Rational(1), 
                  sage.all.ZZ,
                  sage.all.QQ,
                  sage.all.AA,
                  sage.all.matrix(sage.all.ZZ, 2),
                  sage.all.vector(sage.all.ZZ, 2),
                  sage.all.polytopes.cube()]:
            self.assertTrue(is_sage_type(type(a)))

    def test_sage_polymake_sage(self):
        if sage is None:
            return

        # round trip: sage -> polymake -> sage
        for obj in [sage.all.Integer(1),
                    sage.all.Integer(2**32),
                    sage.all.Rational((-3, 5)),
                    sage.all.vector(sage.all.ZZ, [1,2,3]),
                    sage.all.vector(sage.all.QQ, [1,2,sage.all.QQ((3,5))]),
                    sage.all.polytopes.cube(),
                    sage.all.polytopes.associahedron(['B',4]),
                    sage.all.Polyhedron(vertices=[(0,0)], rays=[(1,0),(1,1)]),
                    sage.all.polytopes.dodecahedron()]:

            self.assertEqual(polymake.polymake(obj).sage(), obj)

    def test_polymake_sage_polymake(self):
        if sage is None:
            return

        # round trip: polymake -> sage -> polymake
        # TODO: polymake comparison for polytope is not what you might expect...
        # how do we compare them? (eg, vertices are not sorted)
        for obj in [polymake.Integer(1),
                    polymake.Integer(2**32),
                    polymake.Rational(-3, 5),
                    polymake.VectorInteger([1,2,3]),
                    polymake.VectorRational([1, (2,3), -5])]:

            self.assertEqual(polymake.polymake(obj.sage()), obj)

if __name__ == '__main__':
    unittest.main()
